@@ -17,7 +17,24 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+/* AIX requires this to be the first thing in the file.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
+#endif
 
 /* compatibility define needed for 'gcc -ansi -pedantic' on db.h */
 #ifndef _BSD_SOURCE
@@ -35,10 +52,6 @@
 #include <ctype.h>
 #include <errno.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
 #ifdef HAVE_DB_185_H
 # include <db_185.h>
 #else
@@ -52,6 +65,10 @@
 #include "gnomesupport.h"
 #endif
 
+#if !defined getc_unlocked && !defined HAVE_GETC_UNLOCKED
+# define getc_unlocked(fp) getc (fp)
+#endif
+
 /* Each key in the database has 3 parts: the space, the object, and
    the "key" (confusingly named, I know).  The space is "type" for
    type maps, "file" for direct file maps, and "regex" for regular
@@ -554,7 +571,7 @@ scan_app_file (const struct dirent *ent)
 	if (! line)
 		line = g_string_sized_new (100);
 	equals = comment = column = was_space = skipping = 0;
-	while ((c = getc (f)) != EOF) {
+	while ((c = getc_unlocked (f)) != EOF) {
 		if (c == '\r')
 			continue;
 
@@ -1016,7 +1033,7 @@ got_type:
  * @file: File name
  * @name: Metadata key
  * @size: Return parameter for size of data
- * @data: Return parameter for data
+ * @buffer: Return parameter for data
  *
  * Get a piece of metadata associated with @file.  @size and @buffer
  * are result parameters.  *@buffer is g_malloc()d.
@@ -1039,7 +1056,7 @@ gnome_metadata_get (const char *file, const char *name,
  * @file: File name
  * @name: Metadata key
  * @size: Return parameter for size of data
- * @data: Return parameter for data
+ * @buffer: Return parameter for data
  *
  * Like gnome_metadata_get(), but won't run the `file' command to
  * characterize the file type.
@@ -1145,7 +1162,7 @@ gnome_metadata_copy (const char *from, const char *to)
 }
 
 /**
- * gnome_metadata_copy:
+ * gnome_metadata_delete:
  * @file: File name
  *
  * This function deletes all metadata associated with @file.
@@ -1170,7 +1187,6 @@ gnome_metadata_delete (const char *file)
  * Add a regular expression to the internal list.  This regex is used
  * when matching requests for the metadata @key.
  *
- * Returns nothing.
  */
 void
 gnome_metadata_regex_add (const char *regex, const char *key,
@@ -1186,7 +1202,6 @@ gnome_metadata_regex_add (const char *regex, const char *key,
  *
  * Remove the regular expression from the internal list.
  *
- * Returns nothing.
  */
 void
 gnome_metadata_regex_remove (const char *regex, const char *key)
@@ -1204,7 +1219,6 @@ gnome_metadata_regex_remove (const char *regex, const char *key)
  * Add a file type to the internal list.  This pairing is used
  * when matching requests for the metadata @key.
  *
- * Returns nothing.
  */
 void
 gnome_metadata_type_add (const char *type, const char *key,
@@ -1220,7 +1234,6 @@ gnome_metadata_type_add (const char *type, const char *key,
  *
  * Remove a type/key pairing from the internal list.
  *
- * Returns nothing.
  */
 void
 gnome_metadata_type_remove (const char *type, const char *key)
