@@ -236,8 +236,9 @@ icon_selected_cb(GtkButton * button, GnomeIconEntry * ientry)
 static void
 gil_icon_selected_cb(GnomeIconList *gil, gint num, GdkEvent *event, GnomeIconEntry *ientry)
 {
+	icon_selected_cb(NULL, ientry);
+
 	if(event && event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *)event)->button == 1) {
-		icon_selected_cb(NULL, ientry);
 		gtk_widget_hide(ientry->pick_dialog);
 	}
 }
@@ -248,6 +249,10 @@ show_icon_selection(GtkButton * b, GnomeIconEntry * ientry)
 	GnomeFileEntry *fe = GNOME_FILE_ENTRY(ientry->fentry);
 	char *p = gnome_file_entry_get_full_path(fe,FALSE);
 	char *curfile = gnome_icon_entry_get_filename(ientry);
+	GtkWidget *tl;
+
+	/* Are we part of a modal window?  If so, we need to be modal too. */
+	tl = gtk_widget_get_toplevel (GTK_WIDGET (b));
 	
 	if(!p) {
 		if(fe->default_path)
@@ -290,6 +295,11 @@ show_icon_selection(GtkButton * b, GnomeIconEntry * ientry)
 					 GNOME_STOCK_BUTTON_OK,
 					 GNOME_STOCK_BUTTON_CANCEL,
 					 NULL);
+		if (GTK_WINDOW (tl)->modal) {
+			gtk_window_set_modal (GTK_WINDOW (ientry->pick_dialog), TRUE);
+			gnome_dialog_set_parent (GNOME_DIALOG (ientry->pick_dialog), GTK_WINDOW (tl)); 
+			gnome_file_entry_set_modal (GNOME_FILE_ENTRY (ientry->fentry), TRUE);
+		}
 		gnome_dialog_close_hides(GNOME_DIALOG(ientry->pick_dialog), TRUE);
 		gnome_dialog_set_close  (GNOME_DIALOG(ientry->pick_dialog), TRUE);
 
@@ -302,8 +312,10 @@ show_icon_selection(GtkButton * b, GnomeIconEntry * ientry)
 						   ientry->pick_dialog_dir);
 
 
-		gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(ientry->pick_dialog)->vbox),
-				  iconsel);
+		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(ientry->pick_dialog)->vbox),
+				   ientry->fentry, TRUE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(ientry->pick_dialog)->vbox),
+				   iconsel, TRUE, TRUE, 0);
 
 		gtk_widget_show_all(ientry->pick_dialog);
 
@@ -359,6 +371,7 @@ gnome_icon_entry_init (GnomeIconEntry *ientry)
 {
 	static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
 	GtkWidget *w;
+	GtkWidget *align;
 	char *p;
 
 	gtk_box_set_spacing (GTK_BOX (ientry), 4);
@@ -369,7 +382,7 @@ gnome_icon_entry_init (GnomeIconEntry *ientry)
 	ientry->pick_dialog = NULL;
 	ientry->pick_dialog_dir = NULL;
 	
-	w = gtk_hbox_new(FALSE,0);
+	w = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	gtk_widget_show(w);
 	gtk_box_pack_start (GTK_BOX (ientry), w, TRUE, TRUE, 0);
 	ientry->pickbutton = gtk_button_new_with_label(_("No Icon"));
@@ -385,15 +398,14 @@ gnome_icon_entry_init (GnomeIconEntry *ientry)
 	/*FIXME: 60x60 is just larger then default 48x48, though icon sizes
 	  are supposed to be selectable I guess*/
 	gtk_widget_set_usize(ientry->pickbutton,60,60);
-	gtk_box_pack_start (GTK_BOX (w), ientry->pickbutton,
-			    TRUE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (w), ientry->pickbutton);
 	gtk_widget_show (ientry->pickbutton);
 
 	ientry->fentry = gnome_file_entry_new (NULL,NULL);
 	gtk_signal_connect_after(GTK_OBJECT(ientry->fentry),"browse_clicked",
 				 GTK_SIGNAL_FUNC(browse_clicked),
 				 ientry);
-	gtk_box_pack_start (GTK_BOX (ientry), ientry->fentry, FALSE, FALSE, 0);
+
 	gtk_widget_show (ientry->fentry);
 	
 	p = gnome_pixmap_file(".");
@@ -512,6 +524,26 @@ gnome_icon_entry_set_pixmap_subdir(GnomeIconEntry *ientry,
 	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(ientry->fentry),p);
 	g_free(p);
 }
+/**
+ * gnome_icon_entry_set_icon:
+ * @filename: 
+ * 
+ * Description: Sets the icon of GnomeIconEntry to be the one pointed to by
+ * filename.
+ *
+ * Returns:
+ **/
+void
+gnome_icon_entry_set_icon(GnomeIconEntry *ientry,
+			  const char *filename)
+{
+	g_return_if_fail (ientry != NULL);
+	g_return_if_fail (GNOME_IS_ICON_ENTRY (ientry));
+
+	/* Setting the text in the entry will actually set the icon */
+	gtk_entry_set_text (GTK_ENTRY (gnome_icon_entry_gtk_entry (ientry)),
+			    filename);
+}
 
 /**
  * gnome_icon_entry_get_filaname:
