@@ -646,7 +646,7 @@ icon_new_from_imlib (GnomeIconList *gil, GdkImlibImage *im, const char *text)
 	gnome_icon_text_item_configure (
 		icon->text,
 		0, 0, gil->icon_width, NULL,
-		text, gil->is_editable);
+		text, gil->is_editable, gil->static_text);
 
 	icon->data = NULL;
 	icon->destroy = NULL;
@@ -1071,15 +1071,28 @@ static void
 gil_realize (GtkWidget *widget)
 {
 	Gil *gil = GIL (widget);
+	GtkStyle *style;
 
 	gil->frozen++;
+
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize)(widget);
+
 	gil->frozen--;
 	
+	/* Change the style to use the base color as the background */
+
+	style = gtk_style_copy (gtk_widget_get_style (widget));
+	style->bg[GTK_STATE_NORMAL] = style->base[GTK_STATE_NORMAL];
+	gtk_widget_set_style (widget, style);
+
+	gdk_window_set_background (GTK_LAYOUT (gil)->bin_window,
+				   &widget->style->bg[GTK_STATE_NORMAL]);
+
 	if (gil->frozen)
 		return;
-	if (gil->dirty){
+
+	if (gil->dirty) {
 		gil_layout_all_icons (gil);
 		gil_scrollbar_adjust (gil);
 	}
@@ -1141,6 +1154,9 @@ gil_get_icons_in_region (Gil *gil, double x1, double y1, double x2, double y2)
 	GList *icons, *l;
 
 	icons = NULL;
+
+	if (x1 == x2 || y1 == y2)
+		return icons;
 	
 	for (l = gil->icon_list; l; l = l->next){
 		Icon *icon = l->data;
@@ -1148,19 +1164,17 @@ gil_get_icons_in_region (Gil *gil, double x1, double y1, double x2, double y2)
 		GnomeCanvasItem *text = GNOME_CANVAS_ITEM (icon->text);
 		
 		if (touches_item (image, x1, y1, x2, y2) ||
-		    touches_item (text,  x1, y1, x2, y2)){
+		    touches_item (text,  x1, y1, x2, y2))
 			icons = g_list_prepend (icons, icon);
-
-		}
 	}
 	
-	return icons;
+	return g_list_reverse (icons); /* return it in sorted order */
 }
 
 static void
 gil_mark_region (Gil *gil, GdkEvent *event, double x, double y)
 {
-	GList *icons, *l;
+	GList *icons, *i, *l;
 	double x1, x2, y1, y2;
 	int idx;
 	
@@ -1190,16 +1204,16 @@ gil_mark_region (Gil *gil, GdkEvent *event, double x, double y)
 
 	icons = gil_get_icons_in_region (gil, x1, y1, x2, y2);
 		
-	for (l = gil->icon_list, idx = 0; l; l = l->next, idx++){
+	for (l = gil->icon_list, i = icons, idx = 0; l; l = l->next, idx++){
 		Icon *icon = l->data;
 
-		if (icons && g_list_find (icons, icon)){
+		if (i && i->data == icon){
 			if (!icon->text->selected)
 				gtk_signal_emit (
 					GTK_OBJECT (gil),
 					gil_signals [SELECT_ICON],
 					idx, event);
-			
+			i = i->next;
 		} else if (icon->text->selected){
 			int deselect = FALSE;
 			
@@ -1501,7 +1515,7 @@ gil_class_init (GilClass *gil_class)
 	
 	widget_class->size_request         = gil_size_request;
 	widget_class->size_allocate        = gil_size_allocate;
-	widget_class->realize              = gil_realize; 
+	widget_class->realize              = gil_realize;
 	widget_class->button_press_event   = gil_button_press;
 	widget_class->button_release_event = gil_button_release;
 	widget_class->motion_notify_event  = gil_motion_notify;
@@ -1660,14 +1674,15 @@ gnome_icon_list_set_vadjustment (GnomeIconList *gil, GtkAdjustment *vadj)
 }
 
 void           
-gnome_icon_list_construct (GnomeIconList *gil, guint icon_width, GtkAdjustment *adj, gboolean is_editable)
+gnome_icon_list_construct (GnomeIconList *gil, guint icon_width, GtkAdjustment *adj, int flags)
 {
 	g_return_if_fail (gil != NULL);
 	g_return_if_fail (IS_GIL (gil));
 
 	gnome_icon_list_set_icon_width (gil, icon_width);
-	gil->is_editable = is_editable;
-
+	gil->is_editable = (flags & GNOME_ICON_LIST_IS_EDITABLE) != 0;
+	gil->static_text = (flags & GNOME_ICON_LIST_STATIC_TEXT) != 0;
+	
 	if (!adj)
 		adj = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 1, 0.1, 0.1, 0.1));
 			
@@ -1677,10 +1692,10 @@ gnome_icon_list_construct (GnomeIconList *gil, guint icon_width, GtkAdjustment *
 
 
 /**
- * gnome_icon_list_new: [constructor]
- * @icon_width:  Icon width.
- * @adj:         Scrolling adjustment.
- * @is_editable: whether editing changes can be made to this icon_list.
+ * gnome_icon_list_new_flags: [constructor]
+ * @icon_width: Icon width.
+ * @adj:        Scrolling adjustment.
+ * @flags:      flags that control the icon list creation
  *
  * Creates a new GnomeIconList widget.  Icons will be assumed to be at 
  * most ICON_WIDTH pixels of width.  Any text displayed for those icons
@@ -1693,15 +1708,18 @@ gnome_icon_list_construct (GnomeIconList *gil, guint icon_width, GtkAdjustment *
  * Applications can use this adjustment stored inside the
  * GnomeIconList structure to construct scrollbars if they so desire.
  *
- * If IS_EDITABLE is TRUE, then the text on the icons will be permited to
- * be edited.  If the name changes the "text_changed" signal will be emitted.
+ * if flags has the GNOME_ICON_LIST_IS_EDITABLE flag set, then the
+ * text on the icons will be permited to be edited.  If the name
+ * changes the "text_changed" signal will be emitted.
+ *
+ * if flags has the GNOME_ICON_LIST_STATIC_TEXT flags set, then the
+ * text 
  *
  * Please note that the GnomeIconList starts life in Frozen state.  You are
  * supposed to fall gnome_icon_list_thaw on it as soon as possible.
- *
- */
+ * */
 GtkWidget *
-gnome_icon_list_new (guint icon_width, GtkAdjustment *adj, gboolean is_editable)
+gnome_icon_list_new_flags (guint icon_width, GtkAdjustment *adj, int flags)
 {
 	Gil *gil;
 
@@ -1711,11 +1729,17 @@ gnome_icon_list_new (guint icon_width, GtkAdjustment *adj, gboolean is_editable)
 	gtk_widget_pop_visual ();
 	gtk_widget_pop_colormap ();
 	
-	gnome_icon_list_construct (gil, icon_width, adj, is_editable);
+	gnome_icon_list_construct (gil, icon_width, adj, flags);
 
 	return GTK_WIDGET (gil);
 }
 
+GtkWidget *
+gnome_icon_list_new (guint icon_width, GtkAdjustment *adj, int flags)
+{
+	return gnome_icon_list_new_flags (icon_width, adj, flags & GNOME_ICON_LIST_IS_EDITABLE);
+}
+
 /**
  * gnome_icon_list_freeze:
  * @gil:  The GnomeIconList
@@ -2073,28 +2097,51 @@ int
 gnome_icon_list_get_icon_at (GnomeIconList *gil, int x, int y)
 {
 	GList *l;
-	double dx, dy;
 	double wx, wy;
+	double dx, dy;
+	int cx, cy;
 	int n;
-	
+	GnomeCanvasItem *item;
+	double dist;
+
 	g_return_val_if_fail (gil != NULL, -1);
 	g_return_val_if_fail (IS_GIL (gil), -1);
 
 	dx = x;
 	dy = y;
-	
+
 	gnome_canvas_window_to_world (GNOME_CANVAS (gil), dx, dy, &wx, &wy);
+	gnome_canvas_w2c (GNOME_CANVAS (gil), wx, wy, &cx, &cy);
 
 	for (n = 0, l = gil->icon_list; l; l = l->next, n++){
 		Icon *icon = l->data;
 		GnomeCanvasItem *image = GNOME_CANVAS_ITEM (icon->image);
 		GnomeCanvasItem *text  = GNOME_CANVAS_ITEM (icon->text);
 
-		if (touches_item (image, wx, wy, wx, wy) ||
-		    touches_item (text, wx, wy, wx, wy))
-			return n;
+		if (wx >= image->x1 && wx <= image->x2 && wy >= image->y1 && wy <= image->y2) {
+			dist = (* GNOME_CANVAS_ITEM_CLASS (image->object.klass)->point) (
+				image,
+				wx, wy,
+				cx, cy,
+				&item);
+
+			if ((int) (dist * GNOME_CANVAS (gil)->pixels_per_unit + 0.5)
+			    <= GNOME_CANVAS (gil)->close_enough)
+				return n;
+		}
+
+		if (wx >= text->x1 && wx <= text->x2 && wy >= text->y1 && wy <= text->y2) {
+			dist = (* GNOME_CANVAS_ITEM_CLASS (text->object.klass)->point) (
+				text,
+				wx, wy,
+				cx, cy,
+				&item);
+
+			if ((int) (dist * GNOME_CANVAS (gil)->pixels_per_unit + 0.5)
+			    <= GNOME_CANVAS (gil)->close_enough)
+				return n;
+		}
 	}
 
 	return -1;
 }
-
