@@ -764,6 +764,19 @@ static const struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_PIXMAP_TEXT_ITALIC, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_italic, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_TEXT_UNDERLINE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_underline, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_strikeout, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+
+	{GNOME_STOCK_PIXMAP_ADD, GNOME_STOCK_PIXMAP_REGULAR, N_("Add"), imlib_add, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_CLEAR, GNOME_STOCK_PIXMAP_REGULAR, N_("Clear"), imlib_clear, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_COLORSELECTOR, GNOME_STOCK_PIXMAP_REGULAR, N_("Select Color"), imlib_colorselector, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_REMOVE, GNOME_STOCK_PIXMAP_REGULAR, N_("Remove"), imlib_remove, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TABLE_BORDERS, GNOME_STOCK_PIXMAP_REGULAR, N_("Table Borders"), imlib_table_borders, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TABLE_FILL, GNOME_STOCK_PIXMAP_REGULAR, N_("Table Fill"), imlib_table_fill, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TEXT_BULLETED_LIST, GNOME_STOCK_PIXMAP_REGULAR, N_("Bulleted List"), imlib_text_bulleted_list, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TEXT_NUMBERED_LIST, GNOME_STOCK_PIXMAP_REGULAR, N_("Numbered List"), imlib_text_numbered_list, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TEXT_INDENT, GNOME_STOCK_PIXMAP_REGULAR, N_("Indent"), imlib_text_indent, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_PIXMAP_TEXT_UNINDENT, GNOME_STOCK_PIXMAP_REGULAR, N_("Un-Indent"), imlib_text_unindent, TIGERT_W, TIGERT_H, TB_W, TB_H},
+
+
 	{GNOME_STOCK_BUTTON_OK, GNOME_STOCK_PIXMAP_REGULAR, N_("OK"), imlib_button_ok, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_APPLY, GNOME_STOCK_PIXMAP_REGULAR, N_("Apply"), imlib_button_apply, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_PIXMAP_REGULAR, N_("Cancel"), imlib_button_cancel, TB_W, TB_H, TB_W, TB_H},
@@ -1735,21 +1748,21 @@ accel_dlg_select(GtkCList *widget, gint row, gint col, GdkEventButton *event)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, 0, 2, 2, 3);
 	gtk_object_set_data(GTK_OBJECT(window), "control", w);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(w),
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				    entry.mod & GDK_CONTROL_MASK);
 
 	w = gtk_check_button_new_with_label("Shift");
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, 0, 2, 3, 4);
 	gtk_object_set_data(GTK_OBJECT(window), "shift", w);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(w),
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				    entry.mod & GDK_SHIFT_MASK);
 
 	w = gtk_check_button_new_with_label("Alt");
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, 0, 2, 4, 5);
 	gtk_object_set_data(GTK_OBJECT(window), "alt", w);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(w),
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
 				    entry.mod & GDK_MOD1_MASK);
 
 	w = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
