@@ -18,6 +18,13 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <config.h>
+
+/* need for 'gcc -ansi -pedantic' under GNU/Linux */
+#ifndef _POSIX_SOURCE
+#  define _POSIX_SOURCE 1
+#endif
+#include <sys/types.h>
+
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
@@ -31,19 +38,20 @@
 
 #include "zvtterm.h"
 
-#ifndef ZVT_NO_TRANSPARENT
 #include <gdk/gdkx.h>
 #include <gdk_imlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <X11/Xos.h>
-#endif /* ZVT_NO_TRANSPARENT */
 
 
 /* define to 'x' to enable copious debug output */
 #define d(x)
 
+/* default font */
+#define DEFAULT_FONT "-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1"
+
 
 /* forward declararations */
 static void zvt_term_init (ZvtTerm *term);
@@ -84,14 +92,13 @@ static void vtx_unrender_selection (struct _vtx *vx);
 static void zvt_term_scroll (ZvtTerm *term, int n);
 
 
-#ifndef ZVT_NO_TRANSPARENT
 /* transparent terminal prototypes */
 static Window     get_desktop_window (Window the_window);
 static Pixmap     get_pixmap_prop (Window the_window, char *prop_id);
 static GdkPixmap *load_pixmap_back (char *file, int shaded);
 static GdkPixmap *create_shaded_pixmap (Pixmap p, int x, int y, int w, int h);
 static void       draw_back_pixmap (GtkWidget *widget, int nx, int ny, int nw, int nh);
-#endif /* ZVT_NO_TRANSPARENT */
+static gint       safe_strcmp(gchar *a, gchar *b);
 
 
 /* static data */
@@ -102,17 +109,17 @@ static void       draw_back_pixmap (GtkWidget *widget, int nx, int ny, int nw, i
 static gushort default_red[] = 
 {0x0000,0xaaaa,0x0000,0xaaaa,0x0000,0xaaaa,0x0000,0xaaaa,
  0x5555,0xffff,0x5555,0xffff,0x5555,0xffff,0x5555,0xffff,
- 0xaaaa, 0x0000};
+ 0x0000, 0xffff};
 
 static gushort default_grn[] = 
 {0x0000,0x0000,0xaaaa,0x5555,0x0000,0x0000,0xaaaa,0xaaaa,
  0x5555,0x5555,0xffff,0xffff,0x5555,0x5555,0xffff,0xffff,
- 0xaaaa, 0x0000};
+ 0x0000, 0xffff};
 
 static gushort default_blu[] = 
 {0x0000,0x0000,0x0000,0x0000,0xaaaa,0xaaaa,0xaaaa,0xaaaa,
  0x5555,0x5555,0x5555,0x5555,0xffff,0xffff,0xffff,0xffff,
- 0xaaaa, 0x0000};
+ 0x0000, 0xffff};
 
 
 /* GTK signals */
@@ -216,17 +223,47 @@ zvt_term_class_init (ZvtTermClass *class)
 static void
 zvt_term_init (ZvtTerm *term)
 {
-  /* create and configure callbacks for the teminal
-   * back-end
-   */
-  term->vx = vtx_new (term);
+  GTK_WIDGET_SET_FLAGS (term, GTK_CAN_FOCUS);
+
+  /* create and configure callbacks for the teminal back-end */
+  term->vx = vtx_new (1, 1, term);
   term->vx->vt.ring_my_bell = zvt_term_bell;
   term->vx->vt.change_my_name = zvt_term_title_changed_raise;
 
-  /* widget initalization */
-  GTK_WIDGET_SET_FLAGS (term, GTK_CAN_FOCUS);
   term->shadow_type = GTK_SHADOW_IN;
   term->term_window = NULL;
+  term->cursor_bar = NULL;
+  term->cursor_dot = NULL;
+  term->cursor_current = NULL;
+  term->font = NULL;
+  term->font_bold = NULL;
+  term->scroll_gc = NULL;
+  term->fore_gc = NULL;
+  term->back_gc = NULL;
+  term->fore_last = 0;
+  term->back_last = 0;
+  term->color_ctx = 0;
+  term->ic = NULL;
+
+  /* background pixmap */
+  term->pixmap_filename = NULL;
+  term->background.pix = NULL;
+  term->background.x = 0;
+  term->background.y = 0;
+  term->background.w = 0;
+  term->background.h = 0;
+
+  /* grid sizing */
+  term->set_grid_size_pending = TRUE;
+  term->grid_width = term->vx->vt.width;
+  term->grid_height = term->vx->vt.height;
+
+  /* input handlers */
+  term->input_id = -1;
+  term->msg_id = -1;
+  term->timeout_id = -1;
+
+  /* bitfield flags */
   term->cursor_on = 0;
   term->cursor_filled = 1;
   term->cursor_blink_state = 0;
@@ -237,23 +274,15 @@ zvt_term_init (ZvtTerm *term)
   term->in_expose = 0;
   term->transparent = 0;
   term->shaded = 0;
-  term->pixmap_filename = NULL;
-  term->background.pix = NULL;
-  term->background.x = term->background.y =
-    term->background.w = term->background.h = 0;
-  
-  /* input handlers */
-  term->input_id = -1;
-  term->msg_id = -1;
-  term->timeout_id = -1;
 
-  zvt_term_set_font_name(
-      term, 
-      "-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1");
+  /* charwidth, charheight set here */
+  zvt_term_set_font_name (term, DEFAULT_FONT);
 
   /* scrollback position adjustment */
-  term->adjustment = 
-    GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 24.0, 1.0, 24.0, 24.0));
+  term->adjustment =
+    GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 1.0, 1.0, 1.0));
+  gtk_object_ref (GTK_OBJECT (term->adjustment));
+  gtk_object_sink (GTK_OBJECT (term->adjustment));
 
   gtk_signal_connect (
       GTK_OBJECT (term->adjustment),
@@ -267,11 +296,6 @@ zvt_term_init (ZvtTerm *term)
       GDK_SELECTION_PRIMARY,
       GDK_SELECTION_TYPE_STRING, 
       0);
-
-  /* resizing */
-  term->set_grid_size_pending = TRUE;
-  term->grid_width = term->vx->vt.width;
-  term->grid_height = term->vx->vt.height;
 }
 
 /**
@@ -368,8 +392,28 @@ zvt_term_destroy (GtkObject *object)
 
   term = ZVT_TERM (object);
 
-  zvt_term_closepty(term);
-  vtx_destroy(term->vx);
+  zvt_term_closepty (term);
+  vtx_destroy (term->vx);
+
+  if (term->font)
+    {
+      gdk_font_unref (term->font);
+      term->font = NULL;
+    }
+
+  if (term->font_bold)
+    {
+      gdk_font_unref (term->font_bold);
+      term->font_bold = NULL;
+    }
+
+  /* release the adjustment */
+  if (term->adjustment)
+    {
+      gtk_signal_disconnect_by_data (GTK_OBJECT (term->adjustment), term);
+      gtk_object_unref (GTK_OBJECT (term->adjustment));
+      term->adjustment = NULL;
+    }
 
   if (term->ic) 
     {
@@ -432,7 +476,6 @@ zvt_term_set_default_color_scheme (ZvtTerm *term)
   zvt_term_set_color_scheme (term, default_red, default_grn, default_blu);
 }
 
-
 void
 zvt_term_set_size (ZvtTerm *term, guint width, guint height)
 {
@@ -446,7 +489,6 @@ zvt_term_set_size (ZvtTerm *term, guint width, guint height)
   gtk_widget_queue_resize (GTK_WIDGET (term));
 }
 
-
 static void
 zvt_term_realize (GtkWidget *widget)
 {
@@ -526,8 +568,10 @@ zvt_term_realize (GtkWidget *widget)
   /* Colors */
   term->fore_gc = gdk_gc_new (term->term_window);
   term->back_gc = gdk_gc_new (term->term_window);
-  term->color_ctx = gdk_color_context_new (gtk_widget_get_visual (GTK_WIDGET (term)),
-					   gtk_widget_get_colormap (GTK_WIDGET (term)));
+  term->color_ctx = 
+      gdk_color_context_new (gtk_widget_get_visual (GTK_WIDGET (term)),
+			     gtk_widget_get_colormap (GTK_WIDGET (term)));
+
   /* Allocate default color set */
   zvt_term_set_default_color_scheme (term);
   
@@ -536,7 +580,7 @@ zvt_term_realize (GtkWidget *widget)
   term->fore_last = -1;
 
   /* input context */
-  if (gdk_im_ready() && !term->ic)
+  if (gdk_im_ready () && !term->ic)
     {
       GdkICAttr attr;
       
@@ -550,10 +594,6 @@ zvt_term_realize (GtkWidget *widget)
 	  g_warning("Can't create input context.");
 	}
     }
-
-  /* FIXME: not sure if this is right or not? */
-  if (!GTK_WIDGET_HAS_FOCUS (widget))
-    gtk_widget_grab_focus (widget);
 }
 
 static void
@@ -613,6 +653,10 @@ zvt_term_map (GtkWidget *widget)
 
       gdk_window_show (widget->window);
       gdk_window_show (term->term_window);
+
+      /* XXX: is this right? --JMP */
+      if (!GTK_WIDGET_HAS_FOCUS (widget))
+	gtk_widget_grab_focus (widget);
     }
 }
 
@@ -655,7 +699,7 @@ zvt_term_focus_in(GtkWidget *widget, GdkEventFocus *event)
 
   /* setup blinking cursor */
   if (term->blink_enabled && term->timeout_id == -1)
-    term->timeout_id = gtk_timeout_add(500, zvt_term_cursor_blink, term);
+    term->timeout_id = gtk_timeout_add (500, zvt_term_cursor_blink, term);
 
   if (term->ic)
     gdk_im_begin (term->ic, term->term_window);
@@ -709,21 +753,19 @@ zvt_term_size_request (GtkWidget      *widget,
 
   if (term->set_grid_size_pending)
     {
-      term->set_grid_size_pending = FALSE;
-
       grid_width = term->grid_width;
       grid_height = term->grid_height;
-
+      
       if (grid_width == 0)
-	grid_width = term->vx->vt.width;
-
+	grid_width = 1;
+      
       if (grid_height == 0)
-	grid_height = term->vx->vt.height;
+	grid_height = 1;
     }
   else
     {
-      grid_width = term->vx->vt.width;
-      grid_height = term->vx->vt.height;
+      grid_width = 1;
+      grid_height = 1;
     }
 
   requisition->width = (grid_width * term->charwidth) + 
@@ -824,15 +866,14 @@ zvt_term_draw (GtkWidget *widget, GdkRectangle *area)
 
       gdk_window_get_size (term->term_window, &width, &height);
 
-      /* FIXME: this doesn't need to be done if we're going
-       * draw a pixmap background! --JMP
-       */
-      gdk_window_clear_area (term->term_window, 0, 0, width, height);
-
-#ifndef ZVT_NO_TRANSPARENT
-      if(term->transparent || term->pixmap_filename)
-	draw_back_pixmap(widget, 0, 0, width, height);
-#endif
+      if (term->transparent || term->pixmap_filename)
+	{
+	  draw_back_pixmap(widget, 0, 0, width, height);
+	}
+      else
+	{
+	  gdk_window_clear_area (term->term_window, 0, 0, width, height);
+	}
 
       vt_update_rect (term->vx, 0, 0,
 		      width / term->charwidth,
@@ -871,15 +912,14 @@ zvt_term_expose (GtkWidget      *widget,
 
       if (event->window == term->term_window)
 	{
-#ifndef ZVT_NO_TRANSPARENT
 	  if(term->transparent || term->pixmap_filename)
 	    draw_back_pixmap(widget,
 			     event->area.x, 
 			     event->area.y,
 			     event->area.width, 
 			     event->area.height);
-#endif
-	  vt_update_rect(
+
+	  vt_update_rect (
               term->vx,
 	      event->area.x / term->charwidth,
 	      event->area.y / term->charheight,
@@ -912,7 +952,6 @@ zvt_term_show_pointer (ZvtTerm *term)
     }
 }
 
-
 /**
  * zvt_term_show_pointer:
  * @term: A &ZvtTerm widget.
@@ -932,7 +971,6 @@ zvt_term_hide_pointer (ZvtTerm *term)
     }
 }
 
-
 /**
  * zvt_term_set_scrollback:
  * @term: A &ZvtTerm widget.
@@ -961,9 +999,12 @@ zvt_term_set_fonts_internal(ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
    * what the queue resize will resize the widget to
    * have the same grid size
    */
-  term->set_grid_size_pending = TRUE;
-  term->grid_width = term->vx->vt.width;
-  term->grid_height = term->vx->vt.height;
+  if (!term->set_grid_size_pending)
+    {
+      term->set_grid_size_pending = TRUE;
+      term->grid_width = term->vx->vt.width;
+      term->grid_height = term->vx->vt.height;
+    }
 
   if (font) 
     {
@@ -990,7 +1031,6 @@ zvt_term_set_fonts_internal(ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
   gtk_widget_queue_resize (GTK_WIDGET (term));
 }
 
-
 /**
  * zvt_term_set_fonts:
  * @term: A &ZvtTerm widget.
@@ -1015,7 +1055,6 @@ zvt_term_set_fonts (ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
   gdk_font_ref (font_bold);
 }
 
-
 /**
  * zvt_term_set_font_name:
  * @term: A &ZvtTerm widget.
@@ -1043,7 +1082,7 @@ zvt_term_set_font_name (ZvtTerm *term, char *name)
   g_return_if_fail (name != NULL);
 
   newname = g_string_new (name);
-  outname = g_string_new (NULL);
+  outname = g_string_new ("");
 
   rest = 0;
   ptr = newname->str;
@@ -1053,15 +1092,16 @@ zvt_term_set_font_name (ZvtTerm *term, char *name)
       if (c == '-')
 	{
 	  count++;
-	  d( printf("scanning (%c) (%d)\n", c, count) );
-	  d( printf("newname = %s ptr = %s\n", newname->str, ptr) );
+
+	  d(printf("scanning (%c) (%d)\n", c, count));
+	  d(printf("newname = %s ptr = %s\n", newname->str, ptr));
 
 	  switch (count) 
 	    {
-/* e.g. "-schumacher-clean-medium-r-normal--10-100-75-75-c-80-iso646.1991-irv"*/
 	    case 3:
 	      ptr[-1] = 0;
 	      break;
+
 	    case 5:
 	      rest = ptr - 1;
 	      break;
@@ -1103,16 +1143,16 @@ static void
 zvt_term_fix_scrollbar (ZvtTerm *term)
 {
   GTK_ADJUSTMENT(term->adjustment)->upper = 
-    term->vx->vt.scrollbacklines + term->vx->vt.height-1;
+    term->vx->vt.scrollbacklines + term->vx->vt.height - 1;
 
   GTK_ADJUSTMENT(term->adjustment)->value = 
     term->vx->vt.scrollbackoffset + term->vx->vt.scrollbacklines;
 
   GTK_ADJUSTMENT(term->adjustment)->page_increment = 
-    term->vx->vt.height-1;
+    term->vx->vt.height - 1;
 
   GTK_ADJUSTMENT(term->adjustment)->page_size =
-    term->vx->vt.height-1;
+    term->vx->vt.height - 1;
 
   gtk_signal_emit_by_name (GTK_OBJECT (term->adjustment), "changed");
 }
@@ -1155,9 +1195,10 @@ zvt_term_button_press (GtkWidget      *widget,
       return FALSE;
 
   /* ignore all control-clicks' at this level */
-  if (event->state & GDK_CONTROL_MASK) {
-    return FALSE;
-  }
+  if (event->state & GDK_CONTROL_MASK) 
+    {
+      return FALSE;
+    }
     
   switch(event->button) {
   case 1:			/* left button */
@@ -1427,7 +1468,8 @@ zvt_term_selection_get (GtkWidget        *widget,
 /* receive a selection */
 /* Signal handler called when the selections owner returns the data */
 static void
-zvt_term_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, guint time)
+zvt_term_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, 
+			     guint time)
 {
   struct _vtx *vx;
   ZvtTerm *term;
@@ -1478,7 +1520,6 @@ zvt_term_cursor_blink(gpointer data)
 
   term = ZVT_TERM (widget);
 
-  d(printf("Cursor blinked\n"));
   term->cursor_blink_state ^= 1;
   vt_cursor_state(data, term->cursor_blink_state);
 
@@ -1508,8 +1549,7 @@ zvt_term_scrollbar_moved (GtkAdjustment *adj, GtkWidget *widget)
 
   term->vx->vt.scrollbackoffset = -line;
 
-  d(printf("scrollbar moved to: %f\n\n", adj->value);
-  printf("scroll offset is %d\n", line));
+  d(printf("zvt_term_scrollbar_moved adj->value=%f\n", adj->value));
 
   /* will redraw if scrollbar moved */
   vt_update (term->vx, UPDATE_SCROLLBACK);
@@ -1816,10 +1856,7 @@ zvt_term_key_press (GtkWidget *widget, GdkEventKey *event)
   return handled;
 }
 
-
-/*
- * dummy default signal handler
- */
+/* dummy default signal handler */
 static void
 zvt_term_child_died (ZvtTerm *term)
 {
@@ -1828,9 +1865,8 @@ zvt_term_child_died (ZvtTerm *term)
 
   /* perhaps we should do something here? */
 }
-/*
- * dummy default signal handler for title_changed
- */
+
+/* dummy default signal handler for title_changed */
 static void
 zvt_term_title_changed (ZvtTerm *term, VTTITLE_TYPE type, char *str)
 {
@@ -1840,9 +1876,7 @@ zvt_term_title_changed (ZvtTerm *term, VTTITLE_TYPE type, char *str)
   /* perhaps we should do something here? */
 }
 
-/*
-  raise the title_changed signal
-*/
+/* raise the title_changed signal */
 static void
 zvt_term_title_changed_raise (void *user_data, VTTITLE_TYPE type, char *str)
 {
@@ -1985,7 +2019,6 @@ zvt_term_feed (ZvtTerm *term, char *text, int len)
   zvt_term_fix_scrollbar (term);
 }
 
-
 static void
 zvt_term_readmsg (gpointer data, gint fd, GdkInputCondition condition)
 {
@@ -2006,23 +2039,6 @@ zvt_term_readmsg (gpointer data, gint fd, GdkInputCondition condition)
   gtk_signal_emit (GTK_OBJECT(term), term_signals[CHILD_DIED]);
 }
 
-
-#ifndef ZVT_NO_TRANSPARENT
-/*** FOR TRANSPARENT TERMINAL SUPPORT ***/
-
-static int
-safe_strcmp(char *a, char *b)
-{
-  if(!a && !b)
-    return 0;
-
-  if(a && b)
-    return strcmp(a,b);
-
-  return 1;
-}
-#endif
-
 /**
  * zvt_term_set_background:
  * @terminal: A &ZvtTerm widget.
@@ -2034,7 +2050,6 @@ void
 zvt_term_set_background (ZvtTerm *terminal, char *pixmap_file, 
 			 int transparent, int shaded)
 {
-#ifndef ZVT_NO_TRANSPARENT
   if (!(zvt_term_get_capabilities (terminal) & ZVT_TERM_PIXMAP_SUPPORT))  
     return; 
 
@@ -2044,7 +2059,7 @@ zvt_term_set_background (ZvtTerm *terminal, char *pixmap_file,
      transparent == terminal->transparent)
     return;
   
-  if(terminal->background.pix)
+  if (terminal->background.pix)
     {
       gdk_pixmap_unref(terminal->background.pix);
       terminal->background.pix = NULL;
@@ -2052,15 +2067,14 @@ zvt_term_set_background (ZvtTerm *terminal, char *pixmap_file,
 
   terminal->transparent = transparent;
   terminal->shaded = shaded;
-  g_free(terminal->pixmap_filename);
+  g_free (terminal->pixmap_filename);
   
-  if(pixmap_file)
-    terminal->pixmap_filename = g_strdup(pixmap_file);
+  if (pixmap_file)
+    terminal->pixmap_filename = g_strdup (pixmap_file);
   else
     terminal->pixmap_filename = NULL;
 
   gtk_widget_queue_draw(GTK_WIDGET(terminal));
-#endif
 }
 
 /*
@@ -2082,12 +2096,14 @@ vt_cursor_state(void *user_data, int state)
   old_state = term->cursor_on;
 
   /* only call vt_draw_cursor if the state has changed */
-  if (old_state ^ state) {
-    if (GTK_WIDGET_DRAWABLE (widget)){
-      vt_draw_cursor(term->vx, state);
-      term->cursor_on = state;
+  if (old_state ^ state)
+    {
+      if (GTK_WIDGET_DRAWABLE (widget))
+	{
+	  vt_draw_cursor(term->vx, state);
+	  term->cursor_on = state;
+	}
     }
-  }
   return old_state;
 }
 
@@ -2111,47 +2127,53 @@ vt_draw_text(void *user_data, int col, int row, char *text, int len, int attr)
 
   vx = term->vx;
 
-  if (attr&VTATTR_BOLD) {
-    or = 8;
-    f=term->font_bold;
-  } else {
-    or = 0;
-    f=term->font;
-  }
+  if (attr & VTATTR_BOLD) 
+    {
+      or = 8;
+      f = term->font_bold;
+    } 
+  else 
+    {
+      or = 0;
+      f = term->font;
+    }
 
   fore = (attr & VTATTR_FORECOLOURM) >> VTATTR_FORECOLOURB;
   back = (attr & VTATTR_BACKCOLOURM) >> VTATTR_BACKCOLOURB;
 
-  if (fore<8)
-    fore|=or;
+  if (fore < 8)
+    fore |= or;
 
   /* set the right colour in the appropriate gc */
   fgc = term->fore_gc;
   bgc = term->back_gc;
 
-  if (term->back_last != back){
-    pen.pixel = term->colors [back];
-    gdk_gc_set_foreground (bgc, &pen);
-    term->back_last = back;
-  }
+  if (term->back_last != back)
+    {
+      pen.pixel = term->colors [back];
+      gdk_gc_set_foreground (bgc, &pen);
+      term->back_last = back;
+    }
   
-  if (term->fore_last != fore){
-    pen.pixel = term->colors [fore];
-    gdk_gc_set_foreground (fgc, &pen);
-    term->fore_last = fore;
-  }
+  if (term->fore_last != fore)
+    {
+      pen.pixel = term->colors [fore];
+      gdk_gc_set_foreground (fgc, &pen);
+      term->fore_last = fore;
+    }
 
   /* for reverse, swap gc's */
-  if (attr & VTATTR_REVERSE){
-    GdkGC *tmp = fgc;
-    int tmp2 = fore;
-
-    fgc = bgc;
-    bgc = tmp;
-
-    fore = back;
-    back = tmp2;
-  }
+  if (attr & VTATTR_REVERSE)
+    {
+      GdkGC *tmp = fgc;
+      int tmp2 = fore;
+      
+      fgc = bgc;
+      bgc = tmp;
+      
+      fore = back;
+      back = tmp2;
+    }
 
   /* optimise: dont 'clear' background if not in 
    * expose, and background colour == window colour
@@ -2161,42 +2183,58 @@ vt_draw_text(void *user_data, int col, int row, char *text, int len, int attr)
    * if the terminal is transparent we must redraw 
    * all the time as we can't optimize in that case
    */
-  if (term->in_expose || vx->back_match==0) {
-#ifndef ZVT_NO_TRANSPARENT
-	  if((!term->transparent && !term->pixmap_filename) || back<17) {
-#endif
-		  gdk_draw_rectangle(term->term_window,
-				     bgc,
-				     1,
-				     col*term->charwidth, row*term->charheight,
-				     len*term->charwidth, term->charheight);
-#ifndef ZVT_NO_TRANSPARENT
-	  } else if(!term->in_expose) {
-		  draw_back_pixmap(widget,
-				   col*term->charwidth, row*term->charheight,
-				   len*term->charwidth, term->charheight);
-	  }
-#endif
-  } else {
-    d(printf("not clearing background in_expose = %d, back_match=%d\n", term->in_expose, vx->back_match));
-    d(printf("txt = '%.*s'\n", len, text));
-  }
-
-  gdk_draw_text(term->term_window,
-		f,
-		fgc,
-		col*term->charwidth, row*term->charheight+term->font->ascent,
-		text, len);
+  if (term->in_expose || vx->back_match == 0)
+    {
+      if ((!term->transparent && !term->pixmap_filename) || back < 17)
+	{
+	  gdk_draw_rectangle (
+              term->term_window,
+	      bgc, 1,
+	      col * term->charwidth,
+	      row * term->charheight,
+	      len * term->charwidth,
+	      term->charheight);
+	  
+	} 
+      else if (!term->in_expose) 
+	{
+	  draw_back_pixmap (
+              widget,
+	      col * term->charwidth,
+	      row * term->charheight,
+	      len * term->charwidth,
+	      term->charheight);
+	}
+    } 
+  else 
+    {
+      d(printf("not clearing background in_expose = %d, back_match=%d\n", 
+	       term->in_expose, vx->back_match));
+      d(printf("txt = '%.*s'\n", len, text));
+    }
+  
+  gdk_draw_text(
+      term->term_window, f, fgc,
+      col * term->charwidth,
+      row * term->charheight + term->font->ascent,
+      text, len);
 
   /* check for underline */
-  if (attr&VTATTR_UNDERLINE) {
-    gdk_draw_line(term->term_window,
-		  fgc,
-		  col*term->charwidth, row*term->charheight+term->font->ascent+1,
-		  (col+len)*term->charwidth, row*term->charheight+term->font->ascent+1);
-  }
+  if (attr&VTATTR_UNDERLINE)
+    {
+      gdk_draw_line(
+          term->term_window, fgc,
+	  col * term->charwidth,
+	  row * term->charheight + term->font->ascent + 1,
+	  (col + len) * term->charwidth,
+	  row * term->charheight + term->font->ascent + 1);
+    }
 }
 
+
+
+
+
 void
 vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
 {
@@ -2211,13 +2249,12 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
   
   term = ZVT_TERM (widget);
 
-#ifndef ZVT_NO_TRANSPARENT
-  if(term->transparent || term->pixmap_filename) {
-	  /* FIXME: bad hack */ /* tell me about it... --JMP */
-	  gtk_widget_queue_draw(widget);
-	  return;
-  }
-#endif
+  if(term->transparent || term->pixmap_filename)
+    {
+      /* FIXME: bad hack */ /* tell me about it... --JMP */
+      gtk_widget_queue_draw(widget);
+      return;
+    }
 
   /* FIXME: check args */
   d(printf("scrolling %d rows from %d, by %d lines\n", count,firstrow,offset));
@@ -2349,16 +2386,14 @@ zvt_term_set_shadow_type(ZvtTerm  *term, GtkShadowType type)
  * Returns: a bitmask of the capabilities
  **/
 guint32
-zvt_term_get_capabilities(ZvtTerm *term)
+zvt_term_get_capabilities (ZvtTerm *term)
 {
   guint32 out = 0;
 
-/*pixmap and transparency support*/
-#ifndef ZVT_NO_TRANSPARENT
-  out |= ZVT_TERM_PIXMAP_SUPPORT;
+  /* pixmap and transparency support */
   if (gdk_imlib_get_visual () == gtk_widget_get_default_visual ())
     out |= ZVT_TERM_PIXMAP_SUPPORT;
-#endif
+
   return out;
 }
 
@@ -2459,22 +2494,13 @@ zvt_term_get_buffer(ZvtTerm *term, int *len, int type, int sx, int sy, int ex, i
   return data;
 }
 
-
-
-
 /******************************/
 /**** TRANSPARENT TERMINAL ****/
 
 
-
-#ifndef ZVT_NO_TRANSPARENT
-
 /* kind of stolen from Eterm and needs heavy cleanup */
-
 static Window desktop_window = None;
 
-
-
 static Window
 get_desktop_window (Window the_window)
 {
@@ -2561,7 +2587,6 @@ get_desktop_window (Window the_window)
   return (desktop_window = None);
 }
 
-
 static Pixmap
 get_pixmap_prop (Window the_window, char *prop_id)
 {
@@ -2628,21 +2653,23 @@ create_shaded_pixmap (Pixmap p, int x, int y, int w, int h)
   GdkImlibImage *iim;
   GdkWindowPrivate pw;
   
-  if(p == None)
+  if (p == None)
     return NULL;
   
-  pw.xwindow = (Window)p;
-  iim = gdk_imlib_create_image_from_drawable((GdkPixmap*)&pw,NULL,
-					     x,y,w,h);
+  pw.xwindow = (Window) p;
+  iim = gdk_imlib_create_image_from_drawable(
+	    (GdkPixmap*) &pw, NULL, x, y, w, h);
+
   if(!iim)
     return NULL;
-  mod.contrast=256;
-  mod.brightness=190;
-  mod.gamma=256;
-  gdk_imlib_set_image_modifier(iim,&mod);
-  gdk_imlib_render(iim, iim->rgb_width,iim->rgb_height);
-  pix = gdk_imlib_move_image(iim);
-  gdk_imlib_destroy_image(iim);
+
+  mod.contrast = 256;
+  mod.brightness = 190;
+  mod.gamma = 256;
+  gdk_imlib_set_image_modifier (iim, &mod);
+  gdk_imlib_render (iim, iim->rgb_width, iim->rgb_height);
+  pix = gdk_imlib_move_image (iim);
+  gdk_imlib_destroy_image (iim);
   
   return pix;
 }
@@ -2727,7 +2754,7 @@ draw_back_pixmap (GtkWidget *widget, int nx, int ny, int nw, int nh)
 	      gdk_pixmap_unref(term->background.pix);
 	    }
 	  
-	  term->background.pix = create_shaded_pixmap (p, x,y,width,height);
+	  term->background.pix = create_shaded_pixmap (p, x, y, width, height);
 	  term->background.x = x;
 	  term->background.y = y;
 	  term->background.w = width;
@@ -2755,4 +2782,14 @@ draw_back_pixmap (GtkWidget *widget, int nx, int ny, int nw, int nh)
     }
 }
 
-#endif /* ZVT_NO_TRANSPARENT */
+static gint
+safe_strcmp(gchar *a, gchar *b)
+{
+  if (!a && !b)
+    return 0;
+
+  if (a && b)
+    return strcmp(a,b);
+
+  return 1;
+}
